@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 """
     E707PD Python CLI Application
-    Rev 0.4pre
+    Rev 0.5
 """
 # External Modules Import
 import subprocess
@@ -25,7 +25,9 @@ import json
 import sqlalchemy
 import sqlalchemy.future
 import sqlalchemy.orm
+import sqlalchemy.exc
 import pkg_resources
+import re
 # Local Modules Import
 import e7epd
 # Import of my fork of the digikey_api package
@@ -177,7 +179,9 @@ class CLI:
     cli_revision = e7epd.__version__
 
     class _HelperFunctionExitError(Exception):
-        pass
+        def __init__(self, data=None):
+            self.extra_data = data
+            super().__init__()
 
     class _NoDigikeyApiError(Exception):
         pass
@@ -206,7 +210,7 @@ class CLI:
     def check_for_dk_api(self):
         if not self.is_digikey_available:
             console.print("[orange]API is not setup[/]")
-            console.print("[orange]Please install it with 'pip install git+https://github.com/Electro707/digikey-api.git@1fd3abec434b87a7c051bfa95487c2bfbd4a7651'[/]")
+            console.print("[orange]Please install it with 'pip install git+https://github.com/Electro707/digikey-api.git@8549f42a1853c9d371c3fb1b0b8d780d405174d8'[/]")
             raise self._NoDigikeyApiError
 
     def check_for_db_config(self):
@@ -279,20 +283,25 @@ class CLI:
             if spec['db_name'] == db_name:
                 return spec
 
-    def _ask_manufacturer_part_number(self, part_db: e7epd.E7EPD.GenericPart, must_already_exist: bool = None) -> str:
-        # Get a list of manufacturer part number to use as type hinting
-        mfr_list = []
-        if must_already_exist is not None:
-            try:
-                mfr_list = part_db.get_all_mfr_part_numb_in_db()
-            except e7epd.EmptyInDatabase:
-                if must_already_exist is True:
-                    console.print("[red]No parts found in database[/]")
-                    raise self._HelperFunctionExitError()
-                else:
-                    mfr_list = []
-        if len(mfr_list) != 0:
-            mfr_part_numb = questionary.autocomplete("Enter the manufacturer part number (or scan a Digikey barcode): ", choices=mfr_list).ask()
+    def _ask_manufacturer_part_number(self, existing_mfr_list: list = None, must_already_exist: bool = None) -> str:
+        """
+        Asks for the manufacturer part number. This function handles type hinting with a given list, checking if the mfgr
+        is a Digikey barcode scan, and raises an error if the entered part number is already in the database or not.
+
+        Args:
+            existing_mfr_list: A list of current manufacturer part numbers to typehint
+            must_already_exist: If a given part number must exist in the `existing_mfr_list` or must not exist
+
+        Returns: The entered manufacturer part number
+        """
+        # Replace default argument
+        if existing_mfr_list is None:
+            existing_mfr_list = []
+        # If the exist
+        # if len(existing_mfr_list) == 0 and must_already_exist is True:
+        #     raise UserWarning("Internal Error: existing_mfr_list is None but must_already_exist is True")
+        if len(existing_mfr_list) != 0:
+            mfr_part_numb = questionary.autocomplete("Enter the manufacturer part number (or scan a Digikey barcode): ", choices=existing_mfr_list).ask()
         else:
             mfr_part_numb = questionary.text("Enter the manufacturer part number (or scan a Digikey barcode): ").ask()
         if mfr_part_numb == '' or mfr_part_numb is None:
@@ -306,16 +315,73 @@ class CLI:
             except KeyboardInterrupt:
                 raise self._HelperFunctionExitError()
         if must_already_exist is True:
-            if mfr_part_numb not in mfr_list:
+            if mfr_part_numb not in existing_mfr_list:
                 console.print("[red]Part must already exist in the database[/]")
-                raise self._HelperFunctionExitError()
+                raise self._HelperFunctionExitError(mfr_part_numb)
         elif must_already_exist is False:
-            if mfr_part_numb in mfr_list:
+            if mfr_part_numb in existing_mfr_list:
                 console.print("[red]Part must not already exist in the database, which it does![/]")
-                raise self._HelperFunctionExitError()
+                raise self._HelperFunctionExitError(mfr_part_numb)
         return mfr_part_numb
 
-    def print_parts_list(self, part_db: e7epd.E7EPD.GenericPart, parts_list: list[e7epd.spec.GenericItem], title):
+    def _ask_for_pcb_parts(self) -> list:
+        """
+        Called when wanting to input parts for a PCB
+        """
+        all_parts_dict = []
+        while 1:
+            new_part = {'part': {}}
+            # Ask for the part itself, what it is and get the type
+            specific_part = questionary.select("Is this a specific or generic part?",  choices=["Specific", "Generic", "Done"]).ask()
+            if specific_part is None:
+                raise self._HelperFunctionExitError()
+            elif specific_part == "Done":
+                console.print("Done adding parts")
+                break
+            elif specific_part == "Specific":
+                mgf = self._ask_manufacturer_part_number(self.db.get_all_mfr_part_numb_in_db(), True)
+                _, part_db = self.db.check_if_already_in_db_by_manuf(mgf)
+                new_part['part']['mfr_part_numb'] = mgf
+                new_part['comp_type'] = part_db.table_name
+            elif specific_part == "Generic":
+                generic_part_db = self.choose_component()
+                new_part['comp_type'] = generic_part_db.table_name
+                for spec_db_name in generic_part_db.table_item_display_order:
+                    if spec_db_name in ['stock', 'mfr_part_numb', 'manufacturer', 'storage', 'comments', 'datasheet']:
+                        continue
+                    # Select an autocomplete choice, or None if there isn't any
+                    autocomplete_choices = self.get_autocomplete_list(spec_db_name, generic_part_db.table_name)
+                    # Get the spec
+                    spec = self.find_spec_by_db_name(generic_part_db.table_item_spec, spec_db_name)
+                    if spec is None:
+                        console.print(
+                            "[red]INTERNAL ERROR: Got None when finding the spec for database name %s[/]" % spec_db_name)
+                        raise self._HelperFunctionExitError()
+                    # Ask the user for that property
+                    try:
+                        val, op = self.ask_for_spec_input_with_operator(generic_part_db, spec, autocomplete_choices)
+                        new_part['part'][spec_db_name] = {'val': val, 'op': op}
+                    except KeyboardInterrupt:
+                        console.print("Did not add part")
+                        raise self._HelperFunctionExitError()
+            else:
+                console.print("[red]Invalid Choice![/]")
+                continue
+            # Ask for quantity
+            try:
+                new_part['quantity'] = int(questionary.text("Enter quantity of this part used on this PCB:").ask())
+            except ValueError:
+                console.print("[red]Invalid quantity[/]")
+                continue
+            new_part['quantity'] = questionary.text("Enter the designator/reference:").ask()
+            all_parts_dict.append(new_part)
+
+        if len(all_parts_dict) == 0:
+            console.print("Added nothing for parts")
+            raise self._HelperFunctionExitError()
+        return all_parts_dict
+
+    def print_parts_list(self, part_db: e7epd.E7EPD.GenericComponent, parts_list: list[e7epd.spec.GenericItem], title):
         """ Function is called when the user wants to print out all parts """
         ta = rich.table.Table(title=title)
         for spec_db_name in part_db.table_item_display_order:
@@ -336,15 +402,63 @@ class CLI:
             ta.add_row(*row)
         console.print(ta)
 
-    def print_all_parts(self, part_db: e7epd.E7EPD.GenericPart):
+    def print_all_parts(self, part_db: e7epd.E7EPD.GenericComponent):
         parts_list = part_db.get_all_parts()
         self.print_parts_list(part_db, parts_list, title="All parts in %s" % part_db.table_name)
 
-    def ask_for_spec_input(self, part_db: e7epd.E7EPD.GenericPart, spec: dict, choices: list = None) -> str:
+    def print_filtered_parts(self, part_db: e7epd.E7EPD.GenericComponent):
+        """
+        Prints all parts in a component database with filtering
+        Args:
+            part_db: The component database to use
+        """
+        choices = [questionary.Choice(title=d['showcase_name'], value=d) for d in part_db.table_item_spec]
+        specs_selected = questionary.checkbox("Select what parameters do you want to search by: ", choices=choices).ask()
+        if specs_selected is None:
+            return
+        if len(specs_selected) == 0:
+            console.print("[red]Must choose something[/]")
+            return
+        search_filter = []
+        for spec in specs_selected:
+            autocomplete_choices = self.get_autocomplete_list(spec['db_name'], part_db.table_name)
+            try:
+                inp, op = self.ask_for_spec_input_with_operator(part_db, spec, autocomplete_choices)
+            except KeyboardInterrupt:
+                console.print("Canceled part lookup")
+                return
+            search_filter.append({'db_name': spec['db_name'], 'val': inp, 'op': op})
+        try:
+            parts_list = part_db.get_sorted_parts(search_filter)
+        except e7epd.EmptyInDatabase:
+            console.print("[red]No filtered parts in the database[/]")
+            return
+        self.print_parts_list(part_db, parts_list, title="All parts in %s" % part_db.table_name)
+
+    def print_parts(self, part_db: e7epd.E7EPD.GenericComponent = None):
+        if part_db is None:
+            part_db = self.choose_component()
+        if part_db.is_database_empty():
+            console.print("[italic red]Sorry, but there are no parts for that component[/]")
+            return
+        all_parts = questionary.confirm("Do you want to filter the parts beforehand?", default=False, auto_enter=True).ask()
+        if all_parts:
+            self.print_filtered_parts(part_db)
+        else:
+            self.print_all_parts(part_db)
+
+    def ask_for_spec_input_with_operator(self, part_db: e7epd.E7EPD.GenericComponent, spec: dict, choices: list = None, operator_allowed: bool = True):
+        if spec['input_type'] == 'parts_json':  # We are handling adding a part, which is seperate from a value
+            try:
+                inp = self._ask_for_pcb_parts()
+            except self._HelperFunctionExitError:
+                raise KeyboardInterrupt()
+            return inp
+        op = '=='
         while 1:
             if spec['db_name'] == 'mfr_part_numb':
                 try:
-                    inp = self._ask_manufacturer_part_number(part_db)
+                    inp = self._ask_manufacturer_part_number(part_db.get_all_mfr_part_numb_in_db())
                 except self._HelperFunctionExitError:
                     raise KeyboardInterrupt()
             else:
@@ -364,6 +478,10 @@ class CLI:
             if inp is not None:
                 # Remove leading and trailing whitespace
                 inp = inp.strip()
+                if operator_allowed:
+                    if inp.startswith(tuple(['<', '<=', '>', '>='])):
+                        op = re.findall(r'\>=|\>|\<=|\<', inp)[0]
+                        inp = re.sub(r'\>=|\>|\<=|\<', "", inp)
                 if spec['shows_as'] == 'engineering':
                     try:
                         inp = EngNumber(inp)
@@ -393,6 +511,10 @@ class CLI:
                     console.print("Inputted value is not a %s" % spec['input_type'])
                     continue
             break
+        return inp, op
+
+    def ask_for_spec_input(self, part_db: e7epd.E7EPD.GenericComponent, spec: dict, choices: list = None):
+        inp, op = self.ask_for_spec_input_with_operator(part_db, spec, choices, operator_allowed=False)
         return inp
 
     @staticmethod
@@ -423,49 +545,36 @@ class CLI:
             autocomplete_choices = e7epd.spec.autofill_helpers_list['passive_packages']
         return autocomplete_choices
 
-    def print_filtered_parts(self, part_db: e7epd.E7EPD.GenericPart):
-        choices = [questionary.Choice(title=d['showcase_name'], value=d) for d in part_db.table_item_spec]
-        specs_selected = questionary.checkbox("Select what parameters do you want to search by: ", choices=choices).ask()
-        if specs_selected is None:
-            return
-        if len(specs_selected) == 0:
-            console.print("[red]Must choose something[/]")
-            return
-        part_filter = part_db.part_type()
-        for spec in specs_selected:
-            autocomplete_choices = self.get_autocomplete_list(spec['db_name'], part_db.table_name)
-            try:
-                inp = self.ask_for_spec_input(part_db, spec, autocomplete_choices)
-            except KeyboardInterrupt:
-                console.print("Canceled part lookup")
-                return
-            setattr(part_filter, spec['db_name'], inp)
-        try:
-            parts_list = part_db.get_sorted_parts(part_filter)
-        except e7epd.EmptyInDatabase:
-            console.print("[red]No filtered parts in the database[/]")
-            return
-        self.print_parts_list(part_db, parts_list, title="All parts in %s" % part_db.table_name)
+    def get_partdb_and_mfg(self, part_db: e7epd.E7EPD.GenericComponent = None, mfg_must_exist: bool = None):
+        """
+        Helper function to get the manufacturer part number and component database
+        Args:
+            part_db: The selected component database. If not given, this function will return one depending on the
+                     manufacturer part number
+            mfg_must_exist: Whether the manufacturer part number must already exist in the database
 
-    def get_partdb_and_mfg(self, part_db: e7epd.E7EPD.GenericPart = None, mfg_must_exist: bool = None):
-        mfr_part_numb = None
+        Returns: A tupple of a selected component database and the manufacturer part number
+        """
         if part_db is None:
-            try:
-                part_db = self.choose_component()
-            except KeyboardInterrupt:
-                raise self._HelperFunctionExitError()
-        # Ask for manufacturer part number first, and make sure there are no conflicts
-        if mfr_part_numb is None:
-            try:
-                mfr_part_numb = self._ask_manufacturer_part_number(part_db, must_already_exist=mfg_must_exist)
-            except self._HelperFunctionExitError:
-                raise self._HelperFunctionExitError()
+            all_component_dict = self.db.get_all_component_part_number()
+            all_mfr_list = [item for lists in all_component_dict.values() for item in lists]
+        else:
+            all_mfr_list = part_db.get_all_mfr_part_numb_in_db()
+
+        try:
+            mfr_part_numb = self._ask_manufacturer_part_number(all_mfr_list, must_already_exist=mfg_must_exist)
+        except self._HelperFunctionExitError:
+            raise self._HelperFunctionExitError()
+        if part_db is None:
+            for c in all_component_dict:
+                if mfr_part_numb in all_component_dict[c]:
+                    part_db = self.db.components[c]
+                    break
         return part_db, mfr_part_numb
 
-    def add_new_part(self, part_db: e7epd.E7EPD.GenericPart = None):
+    def add_new_part(self, part_db: e7epd.E7EPD.GenericComponent = None):
         """ Function gets called when a part is to be added """
         try:
-
             if part_db is None:
                 try:
                     part_db = self.choose_component()
@@ -475,8 +584,11 @@ class CLI:
                     return
             if 'mfr_part_numb' in part_db.table_item_display_order:
                 try:
-                    mfr_part_numb = self._ask_manufacturer_part_number(part_db, must_already_exist=False)
-                except self._HelperFunctionExitError:
+                    mfr_part_numb = self._ask_manufacturer_part_number(part_db.get_all_mfr_part_numb_in_db(), must_already_exist=False)
+                except self._HelperFunctionExitError as e:
+                    if e.extra_data is not None:
+                        if questionary.confirm("Would you like to instead add the parts to your stock?", auto_enter=False, default=False).ask():
+                            self.add_stock_to_part(part_db, e.extra_data)
                     return
                 new_part = part_db.part_type(mfr_part_numb=mfr_part_numb)
             else:
@@ -492,7 +604,7 @@ class CLI:
                 if spec is None:
                     console.print("[red]INTERNAL ERROR: Got None when finding the spec for database name %s[/]" % spec_db_name)
                     return
-                # Ask the suer for that property
+                # Ask the user for that property
                 try:
                     setattr(new_part, spec_db_name, self.ask_for_spec_input(part_db, spec, autocomplete_choices))
                 except KeyboardInterrupt:
@@ -504,13 +616,13 @@ class CLI:
             console.print("\nOk, no part is added")
             return
 
-    def delete_part(self, part_db: e7epd.E7EPD.GenericPart):
+    def delete_part(self, part_db: e7epd.E7EPD.GenericComponent):
         """ This gets called when a part is to be deleted """
         try:
             part_db, mfr_part_numb = self.get_partdb_and_mfg(part_db, True)
         except self._HelperFunctionExitError:
             return
-        if questionary.confirm("ARE YOYU SURE...AGAIN???", auto_enter=False, default=False).ask():
+        if questionary.confirm("ARE YOU SURE...AGAIN???", auto_enter=False, default=False).ask():
             try:
                 part_db.delete_part_by_mfr_number(mfr_part_numb)
             except e7epd.EmptyInDatabase:
@@ -520,52 +632,68 @@ class CLI:
         else:
             console.print("Did not delete the part, it is safe.")
 
-    def add_stock_to_part(self, part_db: e7epd.E7EPD.GenericPart = None):
+    def add_stock_to_part(self, part_db: e7epd.E7EPD.GenericComponent = None, mfr_part_numb: str = None):
         try:
-            try:
-                part_db, mfr_part_numb = self.get_partdb_and_mfg(part_db, True)
-            except self._HelperFunctionExitError:
-                return
+            if mfr_part_numb is None or part_db is None:        # If we did not pass a pre-selected mfg part number and part db, ask for it
+                try:
+                    part_db, mfr_part_numb = self.get_partdb_and_mfg(part_db, True)
+                except self._HelperFunctionExitError:
+                    return
+            component = part_db.get_part_by_mfr_part_numb(mfr_part_numb)
+            console.print('There are {:d} parts of the selected component'.format(component.stock))
             while 1:
                 add_by = questionary.text("Enter how much you want to add this part by: ").ask()
+                if add_by is None:
+                    raise KeyboardInterrupt()
                 try:
                     add_by = int(add_by)
                 except ValueError:
                     console.print("Must be an integer")
                     continue
+                if add_by < 0:
+                    console.print("Must be greater than 0")
+                    continue
                 break
-            new_s = part_db.append_stock_by_manufacturer_part_number(mfr_part_numb=mfr_part_numb, append_by=add_by)
-            console.print('[green]Add to your stock :). There is now {:d} left of it.[/]'.format(new_s))
+            component.stock += add_by
+            part_db.commit()
+            console.print('[green]Add to your stock :). There is now {:d} left of it.[/]'.format(component.stock))
         except KeyboardInterrupt:
             console.print("\nOk, no stock is changed")
             return
 
-    def remove_stock_from_part(self, part_db: e7epd.E7EPD.GenericPart = None):
+    def remove_stock_from_part(self, part_db: e7epd.E7EPD.GenericComponent = None):
         try:
             try:
                 part_db, mfr_part_numb = self.get_partdb_and_mfg(part_db, True)
             except self._HelperFunctionExitError:
                 return
+            component = part_db.get_part_by_mfr_part_numb(mfr_part_numb)
+            console.print('There are {:d} parts of the selected component'.format(component.stock))
             while 1:
                 remove_by = questionary.text("Enter how many components to remove from this part?: ").ask()
+                if remove_by is None:
+                    raise KeyboardInterrupt()
                 try:
                     remove_by = int(remove_by)
                 except ValueError:
                     console.print("Must be an integer")
                     continue
+                if remove_by < 0:
+                    console.print("Must be greater than 0")
+                    continue
                 break
-            try:
-                new_s = part_db.remove_stock_by_manufacturer_part_number(mfr_part_numb=mfr_part_numb, remove_by=remove_by)
-            except e7epd.NegativeStock as e_v:
+            if component.stock - remove_by < 0:
                 console.print("[red]Stock will go to negative[/]")
-                console.print("[red]If you want to make the stock zero, restart this operation and remove {:d} parts instead[/]".format(e_v.amount_to_make_zero))
-            else:
-                console.print('[green]Removed to your stock :). There is now {:d} left of it.[/]'.format(new_s))
+                console.print("[red]If you want to make the stock zero, restart this operation and remove {:d} parts instead[/]".format(component.stock))
+                return
+            component.stock -= remove_by
+            part_db.commit()
+            console.print('[green]Removed to your stock :). There is now {:d} left of it.[/]'.format(component.stock))
         except KeyboardInterrupt:
-            console.print("\nOk, no stock is changed")
+            console.print("Ok, no stock is changed")
             return
 
-    def edit_part(self, part_db: e7epd.E7EPD.GenericPart):
+    def edit_part(self, part_db: e7epd.E7EPD.GenericComponent = None):
         """
         Function to update the part's properties
         """
@@ -574,7 +702,7 @@ class CLI:
         try:
             # Ask for manufacturer part number first, and make sure there are no conflicts
             try:
-                mfr_part_numb = self._ask_manufacturer_part_number(part_db, must_already_exist=True)
+                mfr_part_numb = self._ask_manufacturer_part_number(part_db.get_all_mfr_part_numb_in_db(), must_already_exist=True)
             except self._HelperFunctionExitError:
                 return
             part = part_db.get_part_by_mfr_part_numb(mfr_part_numb)
@@ -601,15 +729,75 @@ class CLI:
                 try:
                     setattr(part, to_change['db_name'], self.ask_for_spec_input(part_db, to_change, self.get_autocomplete_list(to_change['db_name'], part_db.table_name)))
                 except KeyboardInterrupt:
-                    console.print("Did not change part")
-                    return
-
+                    console.print("Did not change spec")
+                    continue
         except KeyboardInterrupt:
             part_db.rollback()
-            console.print("\nOk, no stock is changed")
+            console.print("Did not change part")
             return
 
-    def component_cli(self, part_db: e7epd.E7EPD.GenericPart):
+    def print_pcb_and_component_availability(self):
+        all_boards = self.db.pcbs.get_all_boardnames()
+        if len(all_boards) == 0:
+            console.print("There are no PCBs in the database")
+            return
+        board_name = questionary.autocomplete("Enter the PCB name: ", choices=all_boards).ask()
+        if board_name is None:
+            console.print("No board is given")
+            return
+        if board_name not in all_boards:
+            console.print("A board not in the database was given")
+            return
+        rev = questionary.autocomplete("Enter the PCB revision: ", choices=self.db.pcbs.get_revision_per_boardname(board_name)).ask()
+
+        try:
+            board = self.db.pcbs.get_by_boardname_and_rev(board_name, rev)
+        except sqlalchemy.exc.NoResultFound:
+            console.print("[red]No board found for the given name and revision[/]")
+            return
+
+        all_parts_in_board = []
+        for board_part in board.parts:
+            part_db_name, part_db = self.db.get_component_by_table_name(board_part['comp_type'])
+            part_description = ""
+            if 'mfr_part_numb' in board_part['part']:
+                part_description = board_part['part']['mfr_part_numb']
+                try:
+                    parts_in_db = [part_db.get_part_by_mfr_part_numb(board_part['part']['mfr_part_numb'])]
+                except sqlalchemy.exc.NoResultFound:
+                    parts_in_db = []
+            else:
+                search_query = []
+                if board_part['comp_type'] == 'resistance':
+                    part_description = self.db.resistors.print_formatted_from_spec(board_part['part'])
+                for sc in board_part['part']:
+                    if type(board_part['part'][sc]) is dict:
+                        search_query.append({'db_name': sc, 'val': board_part['part'][sc]['val'], 'op': board_part['part'][sc]['op']})
+                    else:
+                        search_query.append({'db_name': sc, 'val': board_part['part'][sc]})
+                parts_in_db = part_db.get_sorted_parts(search_query)
+            if len(parts_in_db) == 0:
+                all_parts_in_board.append(([str(board_part['quantity']), board_part['reference'], part_description, part_db_name, '-', '0', '-'], 's'))
+            for p in parts_in_db:
+                all_parts_in_board.append(([str(board_part['quantity']), board_part['reference'], part_description, part_db_name, p.mfr_part_numb, str(p.stock), p.storage], None))
+
+        console.print("You currently have {:d} PCBs available".format(board.stock))
+
+        ta = rich.table.Table(title='All components for {} Rev {}'.format(board_name, rev))
+        ta.add_column("Stock Required")
+        ta.add_column("Reference")
+        ta.add_column("Description")
+        ta.add_column("Component Type")
+        ta.add_column("Available Mfr Part #")
+        ta.add_column("Stock Available")
+        ta.add_column("Component Location")
+        for parts_in_db in all_parts_in_board:
+            ta.add_row(*parts_in_db[0], style=parts_in_db[1])
+        console.print(ta)
+
+        return
+
+    def component_cli(self, part_db: e7epd.E7EPD.GenericComponent):
         """ The CLI handler for components """
         while 1:
             to_do = questionary.select("What do you want to do in this component database? ", choices=["Print parts in DB", "Append Stock", "Remove Stock", "Add Part", "Delete Part", "Edit Part", self.return_formatted_choice]).ask()
@@ -618,14 +806,7 @@ class CLI:
             if to_do == "Return":
                 break
             elif to_do == "Print parts in DB":
-                if part_db.is_database_empty():
-                    console.print("[italic red]Sorry, but there are no parts for that component[/]")
-                    continue
-                all_parts = questionary.confirm("Do you want to filter the parts beforehand?", default=False, auto_enter=False).ask()
-                if all_parts:
-                    self.print_filtered_parts(part_db)
-                else:
-                    self.print_all_parts(part_db)
+                self.print_parts(part_db)
             elif to_do == "Add Part":
                 self.add_new_part(part_db)
             elif to_do == "Delete Part":
@@ -637,23 +818,27 @@ class CLI:
             elif to_do == "Edit Part":
                 self.edit_part(part_db)
 
-    def choose_component(self) -> e7epd.E7EPD.GenericPart:
+    def choose_component(self) -> e7epd.E7EPD.GenericComponent:
         """
         Dialog to choose which component to use.
         Returns: The component class
         Raises _ChooseComponentDigikeyBarcode: If instead of a component by itself a Digikey barcode is scanned
         Raises KeyboardInterrupt: If a component is not chosen
         """
-        component = questionary.select("Select the component you want do things with:", choices=list(self.db.components.keys()) + [self.return_formatted_choice]).ask()
+        pcb_choice = questionary.Choice(title=prompt_toolkit.formatted_text.FormattedText([('purple', 'PCBs')]))
+        component = questionary.select("Select the component you want do things with:", choices=list(self.db.components.keys()) + [pcb_choice, self.return_formatted_choice]).ask()
         if component is None or component == 'Return':
             raise KeyboardInterrupt()
-        part_db = self.db.components[component]
+        elif component == 'PCBs':
+            part_db = self.db.pcbs
+        else:
+            part_db = self.db.components[component]
         return part_db
 
     def wipe_database(self):
-        do_delete = questionary.confirm("ARE YOYU SURE???", auto_enter=False, default=False).ask()
+        do_delete = questionary.confirm("ARE YOU SURE???", auto_enter=False, default=False).ask()
         if do_delete is True:
-            do_delete = questionary.confirm("ARE YOYU SURE...AGAIN???", auto_enter=False, default=False).ask()
+            do_delete = questionary.confirm("ARE YOU SURE...AGAIN???", auto_enter=False, default=False).ask()
             if do_delete is True:
                 console.print("Don't regret this!!!")
                 self.db.wipe_database()
@@ -722,12 +907,16 @@ class CLI:
         try:
             while 1:
                 to_do = questionary.select("Select the component you want do things with:",
-                                           choices=['Add new part', 'Add new stock', 'Remove stock', 'Individual Components View',
-                                                    'Database Setting', 'Digikey API Settings', 'Exit']).ask()
+                                           choices=['Check components for PCB', 'Search Part', 'Add new part', 'Add new stock', 'Remove stock', 'Edit part', 'Individual Components View',
+                                                    'Database Setting', 'Digikey API Settings', 'Exit'], use_shortcuts=True).ask()
                 if to_do is None:
                     raise KeyboardInterrupt()
                 elif to_do == 'Exit':
                     break
+                elif to_do == 'Check components for PCB':
+                    self.print_pcb_and_component_availability()
+                elif to_do == 'Search Part':
+                    self.print_parts()
                 elif to_do == 'Add new part':
                     try:
                         self.add_new_part()
@@ -743,6 +932,11 @@ class CLI:
                         self.remove_stock_from_part()
                     except KeyboardInterrupt:
                         continue
+                elif to_do == 'Edit part':
+                    try:
+                        self.edit_part()
+                    except KeyboardInterrupt:
+                        continue
                 elif to_do == 'Individual Components View':
                     while 1:
                         try:
